import { IConversionResult, IConverter, IConverterCreator } from "./converter";
import { IFileSystem } from "./files";
import { ILogger } from "./logger";
import { IRunOptions } from "./runner";
/**
 * Dependencies to initialize a new instance of the Converter class.
 */
export interface IConverterDependencies {
    /**
     * Converts files to their language output(s).
     */
    converter: IConverter;
    /**
     * Reads and writes files.
     */
    fileSystem: IFileSystem;
    /**
     * Logs information on significant events.
     */
    logger: ILogger;
    /**
     * Language pre-processors, keyed by their language file extensions.
     */
    preprocessors: Map<string, IConverterCreator>;
}
/**
 * Coordinates converting files to their language outputs.
 */
export declare class Coordinator {
    /**
     * Dependencies used for initialization.
     */
    private readonly dependencies;
    /**
     * Initializes a new instance of the Converter class.
     *
     * @param dependencies   Dependencies used for initialization.
     */
    constructor(dependencies: IConverterDependencies);
    /**
     * Transforms a file to its language output.
     *
     * @param filePath   Original file path.
     * @param options   Options for converting files.
     * @returns A Promise for the file's language output.
     */
    convertFile(filePath: string, options: IRunOptions): Promise<IConversionResult>;
    /**
     * Runs a preprocessor on a file if its extension supports one.
     *
     * @param filePath   Original file path.
     * @param options   Options for converting files.
     * @returns A Promise for processed path to the file.
     */
    private preprocessFile;
}
