import { IConversionResult } from "./converter";
import { Coordinator } from "./coordinator";
import { IFileSystem } from "./files";
import { ILogger } from "./logger";
/**
 * Options to convert a set of files.
 */
export interface IRunOptions {
    /**
     * Contents of file paths to convert, keyed by unique file name.
     */
    files: Map<string, string>;
    /**
     * TypeScript configuration project, if provided.
     */
    typescriptConfig?: string;
}
/**
 * Conversion results for a set of files, keyed by file path.
 */
export interface IFileResults {
    [i: string]: IConversionResult;
}
/**
 * Results from converting a set of files.
 */
export interface IRunResults {
    /**
     * Conversion results for the files, keyed by file path.
     */
    fileResults: IFileResults;
}
/**
 * Dependencies to initialize an instance of the Runner class.
 */
export interface IRunnerDependencies {
    /**
     * Coordinates converting files to their language outputs.
     */
    coordinator: Coordinator;
    /**
     * Reads and writes files.
     */
    fileSystem: IFileSystem;
    /**
     * Logs information on significant events.
     */
    logger: ILogger;
}
/**
 * Persistent runner for converting files.
 */
export declare class Runner {
    /**
     * Dependencies used for initialization.
     */
    private readonly dependencies;
    /**
     * Initializes a new instance of the Runner class.
     *
     * @param dependencies   Dependencies to be used for initialization.
     */
    constructor(dependencies: IRunnerDependencies);
    /**
     * Converts a set of files.
     *
     * @param options   Options for converting files.
     * @returns A Promise for converting the files.
     */
    run(options: IRunOptions): Promise<IRunResults>;
    /**
     * Converts a file.
     *
     * @param filePath   Path to the file.
     * @param options   Options for converting files.
     * @returns A Promise for converting the file.
     */
    private readonly runOnFile;
}
